# coding=utf-8
from datetime import datetime

from ka_uts_com.com import Com
from ka_uts_log.log import Log

from typing import Any
TyAny = Any
TyArr = list[Any]
TyDic = dict[Any, Any]
TyStr = str
TnAny = None | TyAny
TnStr = None | TyStr


class Timestamp:

    @staticmethod
    def sh_elapse_time_sec(end: Any, start: TnAny) -> TnAny:
        if start is None:
            return None
        return end.timestamp()-start.timestamp()


class Timer:
    """ Timer Management
    """
    @staticmethod
    def sh_task_id(d_pacmod: TyDic, class_id: TyAny, parms: TnAny, sep: TyStr) -> TyStr:
        """
        Show task id, which is created by the concationation of the following items:
        package, module, class_name and parms if they are defined; the items package
        and module are get from the package-module directory; the item class_name is
        the class_id if its a string, otherwise the attribute __qualname__ is used.
        """
        package = d_pacmod.get('package')
        module = d_pacmod.get('module')
        if isinstance(class_id, str):
            class_name = class_id
        else:
            class_name = class_id.__qualname__
        if not parms:
            parms = ""
        else:
            parms = f" {parms}"
        arr: TyArr = []
        for item in [package, module, class_name, parms]:
            if not item:
                continue
            arr.append(item)
        return sep.join(arr)

    @classmethod
    def start(cls, class_id: TyAny, parms: TnAny = None, sep: TyStr = ".") -> None:
        """ Start Timer
        """
        task_id = cls.sh_task_id(Com.d_app_pacmod, class_id, parms, sep)
        Com.d_timer[task_id] = datetime.now()

    @classmethod
    def end(cls, class_id: TyAny, parms: TnAny = None, sep: TyStr = ".") -> None:
        """ End Timer
        """
        task_id = cls.sh_task_id(Com.d_app_pacmod, class_id, parms, sep)
        start = Com.d_timer.get(task_id)
        end = datetime.now()
        elapse_time_sec = Timestamp.sh_elapse_time_sec(end, start)
        msg = f"{task_id} elapse time [sec] = {elapse_time_sec}"
        Log.info(msg, stacklevel=2)



# coding=utf-8
from typing import Any

TyAny = Any
TyArr = list[Any]
TyBool = bool
TyDic = dict[Any, Any]

TnAny = None | Any
TnArr = None | TyArr
TnBool = None | bool
TnDic = None | TyDic


class Exit:
    """Exit Class
    """
    sw_init: TyBool = False
    sw_critical: bool = False
    sw_stop: bool = False
    sw_interactive: bool = False

    @classmethod
    def init(cls, **kwargs) -> None:
        if cls.sw_init:
            return
        cls.sw_init = True
        cls.sw_critical = kwargs.get('sw_critical', False)
        cls.sw_stop = kwargs.get('sw_stop', False)
        cls.sw_interactive = kwargs.get('sw_interactive', False)

    @classmethod
    def sh(cls, **kwargs) -> Any:
        if cls.sw_init:
            return cls
        cls.init(**kwargs)
        return cls



"""
Decorators
"""
# coding=utf-8
from ka_uts_com.timer import Timer


def timer(fnc):
    def wrapper(*args, **kwargs):
        Timer.start(fnc)
        fnc(*args, **kwargs)
        Timer.end(fnc)
    return wrapper



__title__ = 'ka_uts_com'
__description__ = 'Communication Utilities.'
__url__ = 'https://ka-ut-com.readthedocs.io/en/latest'
__version__ = '3.0.0.250505'
__build__ = 0x022200
__author__ = 'Bernd Stroehle'
__author_email__ = 'bernd.stroehle@gmail.com'
__license__ = 'Apache-2.0'
__copyright__ = 'Copyright 2025 Bernd Stroehle'
__cake__ = u'\u2728 \U0001f370 \u2728'



# coding=utf-8
from typing import Any

from ka_uts_uts.ioc.yaml_ import Yaml_
from ka_uts_uts.utils.pacmod import PacMod

TyAny = Any
TyTimeStamp = int
TyArr = list[Any]
TyBool = bool
TyDic = dict[Any, Any]


class Cfg:
    """Configuration Class
    """
    sw_init: TyBool = False
    cfg: Any = None

    @classmethod
    def init(cls, cls_com, **kwargs) -> None:
        if cls.sw_init:
            return
        cls.sw_init = True
        _cls_app = kwargs.get('cls_app')
        _path = PacMod.sh_path_cfg_yml(_cls_app)
        if _path:
            cls.cfg = Yaml_.read_with_safeloader(_path)

    @classmethod
    def sh(cls, cls_com, **kwargs) -> Any:
        if cls.sw_init:
            return cls
        cls.init(cls_com, **kwargs)
        return cls.cfg



# coding=utf-8
from typing import Any

from ka_uts_log.log import Log
from ka_uts_uts.utils.pacmod import PacMod
from ka_uts_uts.ioc.yaml_ import Yaml_

TyAny = Any
TyArr = list[Any]
TyBool = bool
TyDic = dict[Any, Any]

TnAny = None | Any
TnArr = None | TyArr
TnBool = None | bool
TnDic = None | TyDic


class App:
    """Aplication Class
    """
    sw_init: TyBool = False
    sw_replace_keys: TnBool = None
    keys: TnArr = None
    httpmod: TyAny = None
    reqs: TyDic = {}
    app: TyDic = {}

    @classmethod
    def init(cls, cls_com, **kwargs) -> None:
        if cls.sw_init:
            return
        cls.sw_init = True
        cls.httpmod = kwargs.get('httpmod')
        cls.sw_replace_keys = kwargs.get('sw_replace_keys', False)
        if cls.sw_replace_keys:
            try:
                cls.keys = Yaml_.read_with_safeloader(PacMod.sh_path_keys_yml(cls_com))
            except Exception as exc:
                Log.error(exc, exc_info=True)
                raise

    @classmethod
    def sh(cls, cls_com, **kwargs) -> Any:
        if cls.sw_init:
            return cls
        cls.init(cls_com, **kwargs)
        return cls
